#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/6/12 0:46
@File  : MonitorDriver.py
'''
import getpass
import os
import re
import subprocess
import copy
from lxml import etree
import tkinter as tk
import threading
import time
from adbutils import adb

USERHOMEDIR = f'C:\\Users\\{getpass.getuser()}'

UI_DUMP_FILE = f'{USERHOMEDIR}\\window_dump.xml'

class App:
    def __init__(self, master):
        self.master = master
        self.device_var = tk.StringVar()
        self.app_var = tk.StringVar()
        self.text_var = tk.StringVar()
        self.create_widgets()
        self.start_threads()
        self.master.protocol("WM_DELETE_WINDOW", self.stop_threads)


    def create_widgets(self):
        # 创建下拉框
        self.device_label = tk.Label(self.master, text="设备号:")
        self.device_label.grid(row=0, column=0)
        devices = adb.device_list()
        device_list = [d.serial for d in devices]
        self.device_var.set(device_list[0])
        self.device_option = tk.OptionMenu(self.master, self.device_var, *device_list)
        self.device_option.grid(row=0, column=1)

        # 创建标签
        self.app_label = tk.Label(self.master, text="APP信息:")
        self.app_label.grid(row=1, column=0)
        self.app_info = tk.Entry(self.master, textvariable=self.app_var, width=len(self.app_var.get()))
        self.app_info.grid(row=1, column=1)

        # 创建多行输入框
        self.text_label = tk.Label(self.master, text="点击页面:")
        self.text_label.grid(row=2, column=0)
        self.text_box = tk.Text(self.master, height=30, width=60)
        self.text_box.grid(row=2, column=1)
        self.text_box.insert('1.0', '开始点击你的APP界面吧!')
        self.scrollbar = tk.Scrollbar(self.master, command=self.text_box.yview)
        self.scrollbar.grid(row=2, column=2, sticky="nsew")
        self.text_box.config(yscrollcommand=self.scrollbar.set)

    def start_threads(self):
        # 启动线程
        self.device_thread = threading.Thread(target=self.update_device)
        self.device_thread.start()
        self.app_thread = threading.Thread(target=self.update_app)
        self.app_thread.start()
        self.text_thread = threading.Thread(target=self.update_text)
        self.text_thread.start()
        self.lock = threading.Lock()

    def stop_threads(self):
        # 停止线程
        self.device_thread.do_run = False
        self.app_thread.do_run = False
        self.text_thread.do_run = False
        self.master.destroy()

    def update_device(self):
        self.device_thread.do_run = True
        while self.device_thread.do_run:
            # 获取设备号并更新下拉框
            devices = adb.device_list()
            device_list = [d.serial for d in devices]
            self.device_option['menu'].delete(0, 'end')
            for device in device_list:
                self.device_option['menu'].add_command(label=device, command=tk._setit(self.device_var, device))
            time.sleep(0.5)

    def update_app(self):
        self.app_thread.do_run = True
        while self.app_thread.do_run:
            # 获取APP信息并更新标签
            self.device = adb.device(self.device_var.get())
            self.app = self.device.app_current()
            self.package_name = getattr(self.app, 'package')
            self.activity_name = getattr(self.app, 'activity')
            app_info = f"{self.package_name}/{self.activity_name}"
            self.app_var.set(app_info)
            time.sleep(0.5)
    def update_text(self):
        self.text_thread.do_run = True
        while self.text_thread.do_run:
            # 获取点击页面的内容并更新多行输入框
            device = adb.device(self.device_var.get())
            self.monitor_device(device=device, output_type='appium')

    def find_smallest_bounds_index(self, target, bounds_list):
        """
        查找包含目标点的最小边界的索引。

        参数：
            target (tuple): 目标点格式为 (x, y)。
            bounds_list (list): 边界列表，格式为 [[x0, y0, x1, y1], ...]。

        返回值：
            int: 包含目标点的最小边界的索引。
        """
        smallest_area = float('inf')
        smallest_index = None

        for i, bounds in enumerate(bounds_list):
            x0, y0, x1, y1 = bounds
            if x0 < target[0] < x1 and y0 < target[1] < y1:
                area = (x1 - x0) * (y1 - y0)
                if area < smallest_area:
                    smallest_area = area
                    smallest_index = i

        return smallest_index

    def generate_xpath_with_attributes(self, element):
        if element is None:
            return ''
        if element.tag == 'hierarchy':
            class_value = element.get('class', '')
        else:
            class_value = element.get('class') if element.get('class') != '' else '*'
        resource_id = element.get('resource-id')
        text = element.get('text')

        if text:
            return f'//{class_value}[@text="{text}"]'
        elif resource_id:
            return f'//{class_value}[@resource_id="{resource_id}"]'
        else:
            parent = element.getparent()
            parent_xpath = self.generate_xpath_with_attributes(parent)
            return f'{parent_xpath}/{class_value}'

    # 实时监控设备
    def monitor_device(self, device, output_type: str = 'appium'):
        '''
        监控设备并输入指定类型的数据
        :param driverName: 设备名
        :param output_type: data or appium
        :return:
        '''
        device = adb.device(device.serial)
        app = device.app_current()
        package_name = getattr(app, 'package')
        activity_name = getattr(app, 'activity')
        print(f'''#! /usr/bin/env python

        from appium import webdriver

        caps = {{
            'automationName': 'UiAutomator2',
            'platformName': 'Android',
            'platformVersion': {device.shell(['getprop', 'ro.build.version.release']).strip()},
            'deviceName': {device.serial},
            'appPackage': {package_name},
            'appActivity': {activity_name},
            # 'noReset': True,
            # 'dontStopAppOnReset': True,
            'unicodeKeyboard': True,
            'resetKeyboard': True
        }}

        app = webdriver.Remote("http://localhost:4723/wd/hub", caps)

        # 下面为定位表达式
        ''')
        deviceName = device.serial
        # 启动getevent命令
        event_cmd = f"adb -s {deviceName} shell getevent -lt /dev/input/event1"
        process = subprocess.Popen(event_cmd, stdout=subprocess.PIPE, shell=True)
        x = 0
        y = 0
        dx = copy.copy(x)
        dy = copy.copy(y)
        sj = ''
        status = [None, None]
        down_time = float(0)
        # 解析事件流
        while deviceName == device.serial:
            device = adb.device(device.serial)
            app = device.app_current()
            package_name = getattr(app, 'package')
            activity_name = getattr(app, 'activity')
            code_header = "#! /usr/bin/env python\n" +\
                        "\n" +\
                        "from appium import webdriver\n" +\
                        "\n" +\
                        "caps = {\n" +\
                            "\t'automationName': 'uiautomator2',\n" +\
                            "\t'platformName': 'Android',\n" +\
                            f"\t'platformVersion': '{device.shell(['getprop', 'ro.build.version.release']).strip()}',\n" +\
                            f"\t'deviceName': '{device.serial}',\n" +\
                            "\t# 'noReset': True,\n" +\
                            "\t# 'allowClearUserData': 'true',\n" +\
                            "\t# 'fullReset': \"false\",\n" +\
                            "\t# 'exported': \"true\",\n" +\
                            f"\t'appPackage': '{package_name}',\n" +\
                            f"\t'appActivity': '{activity_name}',\n" +\
                            "\t'unicodeKeyboard': True,\n" +\
                            "\t'resetKeyboard': True\n" +\
                        "}\n" +\
                        "\n" +\
                        "app = webdriver.Remote('http://127.0.0.1:4723/wd/hub', caps)\n" +\
                        "\n" +\
                        "# 以下内容为您当前触屏事件内容：\n"
            # 读取一行事件
            line = process.stdout.readline().decode().strip()
            # 获取x坐标
            if 'POSITION_X' in line:
                parts = line.split()
                x = int(parts[4], 16)

            if 'POSITION_Y' in line:
                parts = line.split()
                y = int(parts[4], 16)

            # 如果是点击事件
            if "BTN_TOUCH" in line and "DOWN" in line:
                down_time = time.time()
                status[0] = 'DOWN'
                # 记录点下时的坐标
                dx = copy.copy(x)
                dy = copy.copy(y)

                # 生成 XML 文件
                # os.popen('adb shell uiautomator dump')
                device.shell(['uiautomator', 'dump'])

            # 如果是点击事件
            if "BTN_TOUCH" in line and "UP" in line:
                hold_time = round(time.time() - down_time, 3)  # 秒
                status[1] = 'UP'
                # 记录点下时的坐标
                ux = copy.copy(x)
                uy = copy.copy(y)

                self.lock.acquire()
                os.popen(f'adb pull /sdcard/window_dump.xml {UI_DUMP_FILE}')
                # device.shell(['pull', '/sdcard/window_dump.xml', UI_DUMP_FILE])
                self.lock.release()
                while not os.path.exists(UI_DUMP_FILE) or os.path.getsize(UI_DUMP_FILE) == 0:
                    if os.path.exists(UI_DUMP_FILE):
                        self.lock.acquire()
                        os.remove(UI_DUMP_FILE)
                        self.lock.release()
                    self.lock.acquire()
                    # 将 XML 文件保存到本地
                    os.popen(f'adb pull /sdcard/window_dump.xml {UI_DUMP_FILE}')
                    # device.shell(['pull', '/sdcard/window_dump.xml', UI_DUMP_FILE])
                    self.lock.release()
                self.lock.acquire()
                # 解析 XML 文档
                tree = etree.parse(UI_DUMP_FILE)
                self.lock.release()  # 在代码块结束后释放锁
                elements = tree.xpath('//node')
                bounds = [[int(num) for num in re.findall(r'\d+', s)] for s in tree.xpath('//node/@bounds')]
                # 防止点击页面以外的坐标，限制x与y的最大值与最小值
                x_page_max = bounds[0][2]
                y_page_max = bounds[0][3]
                ux = x_page_max-1 if ux > x_page_max else ux
                uy = y_page_max-1 if uy > y_page_max else uy
                dx = x_page_max-1 if dx > x_page_max else dx
                dy = y_page_max-1 if dy > y_page_max else dy
                ux = 1 if ux < 0 else ux
                uy = 1 if uy < 0 else uy
                dx = 1 if dx < 0 else dx
                dy = 1 if dy < 0 else dy
                up_index = self.find_smallest_bounds_index((ux, uy), bounds)
                down_index = self.find_smallest_bounds_index((dx, dy), bounds)

                if status == ['DOWN', 'UP']:
                    if up_index == down_index:
                        if output_type == 'data':
                            sj = {
                            'platformName': 'Android',
                            'platformVersion': device.shell(['getprop', 'ro.build.version.release']).strip(),
                            'deviceName': device.serial,
                            'appPackage': getattr(device.app_current, 'package'),
                            'appActivity': getattr(device.app_current, 'activity'),
                            'action': 'click',
                            'xpath': self.generate_xpath_with_attributes(elements[up_index]),
                            'bounds': [dx, dy, ux, uy],
                            'hold_time': hold_time
                        }
                        elif output_type == 'appium':
                            sj = f'app.find_element(By.XPATH, value="{self.generate_xpath_with_attributes(elements[up_index])}").click()'
                        else:
                            sj = '暂不支持此类型'
                    else:
                        if output_type == 'data':
                            sj = {
                                'platformName': 'Android',
                                'platformVersion': device.shell(['getprop', 'ro.build.version.release']).strip(),
                                'deviceName': device.serial,
                                'appPackage': getattr(device.app_current, 'package'),
                                'appActivity': getattr(device.app_current, 'activity'),
                                'action': 'swipe',
                                'xpath': None,
                                'bounds': [dx, dy, ux, uy],
                                'hold_time': hold_time
                            }
                        elif output_type == 'appium':
                            sj = f'app.swipe({dx}, {dy}, {ux}, {uy}, {int(hold_time*1000)})'
                        else:
                            sj = '暂不支持此类型'
                    status = [None, None]
            self.text_box.delete('1.0', 'end')
            self.text_box.insert('1.0', code_header+sj)
            time.sleep(0.1)
        self.monitor_device(device=device.serial)

def gui():
    root = tk.Tk()
    app = App(root)
    root.mainloop()


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/29 0:15
@File  : networkSpeedUtils.py
'''
from speedtest import Speedtest
from saf import Union

def unitConversion(speed: Union[int, float]):
    '''
    网速单位转换
    :param speed        :  网络传输值
    :return             :  (网速单位, 网速转换值, 网速原值, 单位级别)
    '''
    if 0 <= speed < 1024 ** 1:
        return f'{speed} bit/s', speed//(1024 ** 0), speed, 0
    elif 1024 ** 1 <= speed < 1024 ** 2:
        return f'{speed//1024 ** 1} Kbit/s', speed//(1024 ** 1), speed, 1
    elif 1024 ** 2 <= speed < 1024 ** 3:
        return f'{speed // 1024 ** 2} Mbit/s', speed//(1024 ** 2), speed, 2
    elif 1024 ** 3 <= speed < 1024 ** 4:
        return f'{speed // 1024 ** 3} Gbit/s', speed//(1024 ** 3), speed, 3
    elif 1024 ** 4 <= speed < 1024 ** 5:
        return f'{speed // 1024 ** 4} Tbit/s', speed//(1024 ** 4), speed, 4
    elif 1024 ** 5 <= speed < 1024 ** 6:
        return f'{speed // 1024 ** 5} Pbit/s', speed//(1024 ** 5), speed, 5
    else:
        return '您的网速太快的，外星球来的吧~，~'

def testNetworkSpeed():
    test = Speedtest()
    test.get_servers()
    return unitConversion(test.download()), unitConversion(test.upload())

# if __name__ == '__main__':
#     print(testNetworkSpeed())


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:29
@File  : config.py
'''
import hashlib
import time

class feishu(object):
    @staticmethod
    def webhook():
        ''' 飞书webhook的地址 '''
        return 'https://open.feishu.cn/open-apis/bot/v2/hook/xxxx'

class dingding(object):
    @staticmethod
    def webhook():
        ''' 钉钉webhook的地址 '''
        return 'https://oapi.dingtalk.com/robot/send?access_token=xxxxxxxx'

class email(object):
    @staticmethod
    def smtpServer():
        ''' smtp服务器IP或者域名
        163邮箱：http://help.163.com/09/1223/14/5R7P3QI100753VB8.html
        QQ邮箱：https://service.mail.qq.com/cgi-bin/help?subtype=1&id=28&no=369
        '''
        return 'smtp.163.com'

    @staticmethod
    def smtpPort():
        ''' smtp端口： '''
        return 25

    @staticmethod
    def username():
        ''' 邮箱登录账户 '''
        return 'username'

    @staticmethod
    def password():
        ''' 授权码，163和QQ的不是登录密码 '''
        return 'password'

    @staticmethod
    def isSSL():
        ''' 是否使用安全隧道 '''
        return False

class zenTao(object):
    '''
    参考禅道接口文档：
    https://www.zentao.net/book/zentaopmshelp/integration-287.html
    '''
    @staticmethod
    def baseURL():
        ''' 禅道的根路径 '''
        return 'http://192.168.254.133/zentao'

    @staticmethod
    def account():
        ''' 后台-》二次开发-》应用-》免密登录的账户名 '''
        return 'admin'

    @staticmethod
    def getCode():
        ''' 后台-》二次开发-》应用-》创建-》代号 '''
        return 'xbkj'

    @staticmethod
    def getKey():
        ''' 后台-》二次开发-》应用-》创建-》密钥 '''
        return ''

    @staticmethod
    def getTime():
        ''' 获取时间戳 '''
        return int(time.time())

    @staticmethod
    def getToken():
        ''' 获取token： md5($code + $key + $time) '''
        _md5 = hashlib.md5(f'{zenTao.getCode()}{zenTao.getKey()}{zenTao.getTime()}'.encode('utf-8'))
        return _md5.hexdigest()

class Jira(object):
    @staticmethod
    def server_url():
        return ''

    @staticmethod
    def username():
        return ''

    @staticmethod
    def password():
        return ''

class browser(object):
    @staticmethod
    def name():
        ''' 浏览器名称： chrome 或 firefox 或 ie '''
        return 'chrome'

class log(object):
    @staticmethod
    def path():
        ''' 日志存储的路径 '''
        return '.'

    @staticmethod
    def name():
        ''' 日志文件名 '''
        return 'auto_info_{time}.log'

    @staticmethod
    def rotation():
        ''' 日志存储规则：50 MB 或者 1 days '''
        return '1 days'


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/6/16 0:04
@File  : Demo.py
'''
'''
Python3.9+tkinter+adb+matplotlib等库；实现一下功能：用户界面显示连接设备当前APP的实时耗电量，横坐标为Time(s),纵坐标为Powser(%)，横坐标只显示最近的30秒内的耗电量情况，图像使用折线图显示数据，图像下方有三个按钮（开始按钮、停止按钮、导出按钮分别对应三个功能）；请实现完整可用的代码！
'''

import tkinter as tk
from tkinter import ttk
import adbutils
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import numpy as np

# 连接Android设备
adb = adbutils.AdbClient()
device = adb.device()

# 设置GUI
root = tk.Tk()
root.title('Android电量监控')

# 初始化变量
x = []  # 时间轴
y = []  # 电量数据
isRunning = False  # 线程运行控制


# 定义绘图函数
def plot_power():
    global x, y

    # 限制x轴范围为最近30s
    if len(x) > 30:
        x = x[-30:]
        y = y[-30:]

    # 绘图

    plt.xlabel('Time/s')
    plt.ylabel('Power/%')
    print(5)
    # plt.clf()  # 清除之前的图像
    plt.plot(x, y, '-r')

# 更新图形
def update_graph():
    global x, y, isRunning
    if isRunning:
        print(4)
        # 获取当前电量
        output = device.shell('dumpsys battery | grep level')
        power_level = int(output.split(':')[1].strip())
        print(f'powser: {power_level}')
        # 添加新的数据
        x.append(len(x))
        y.append(power_level)

        # 更新图像
        plot_power()
        draw.draw()

        print(6)
        # 每0.5s获取一次数据
        root.after(500, update_graph)


# 开始按钮
def start():
    print(1)
    global isRunning
    print(2)
    isRunning = True
    print(3)
    update_graph()


# 停止按钮
def stop():
    global isRunning
    isRunning = False


# 导出按钮
def export():
    plt.savefig('power.png')


# 绘制图像
fig = plt.Figure()
plot_power()
draw = FigureCanvasTkAgg(fig, master=root)
draw.get_tk_widget().grid(row=0, columnspan=3)

# 添加按钮
tk.Button(root, text="Start", command=start).grid(row=1, column=0)
tk.Button(root, text="Stop", command=stop).grid(row=1, column=1)
tk.Button(root, text="Export", command=export).grid(row=1, column=2)

# 事件循环
root.mainloop()


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/6/26 21:04
@File  : MonitorDBs.py
'''
import os

import mysql.connector  # pip install mysql-connector-python
import sqlite3
import cx_Oracle    # pip install cx_Oracle
from docx import Document   # pip install python-docx

def generate_data_dictionary(database_type, connection_details, save_filename = None):
    '''
    依据数据库存储的数据生成数据字典的Word文件
    :param database_type: sqlite、MySQL、Oracle
    :param connection_details:
    # 实例：

    # mysql/oracle样例：
    connection_details = {
        'host': 'localhost',
        'user': 'username',
        'password': 'password',
        'database': 'database_name'
    }
    generate_data_dictionary('mysql', connection_details, 'xiaobai.docx')

    # sqlite样例：
    connection_details = 'D:\\coding\\studentSystem\\xiaobai_db.db'
    generate_data_dictionary('sqlite', connection_details)

    :return:

    '''
    # 连接数据库
    if database_type == 'mysql':
        connection = mysql.connector.connect(**connection_details)
    elif database_type == 'sqlite':
        connection = sqlite3.connect(connection_details)
    elif database_type == 'oracle':
        connection = cx_Oracle.connect(connection_details)
    else:
        raise ValueError('Unsupported database type')

    # 进度条
    progress_rate = 100.0

    # 生成数据字典的Word文件
    document = Document()

    # 获取数据库信息
    cursor = connection.cursor()
    if 'sqlite' not in database_type:
        if connection_details['database']:
            databases = [[connection_details['database']]]
        else:
            cursor.execute("SHOW DATABASES")  # 获取所有数据库
            databases = cursor.fetchall()

        for i, database in enumerate(databases):
            database_name = database[0]
            document.add_heading(database_name, level=1)

            cursor.execute(f"USE {database_name}")  # 切换到当前数据库
            cursor.execute("SHOW TABLES")  # 获取当前数据库的所有数据表
            tables = cursor.fetchall()

            for j, table in enumerate(tables):
                progress_rate = ((i+1)*(j+1) / (len(databases) * len(tables))) * 100.0
                table_name = table[0]
                document.add_heading(table_name, level=2)

                # cursor.execute(f"DESCRIBE {table_name}")  # 获取数据表结构
                cursor.execute(f"SHOW FULL COLUMNS from {table_name}")  # 获取数据表结构
                table_structure = cursor.fetchall()

                table_info = document.add_table(rows=1, cols=3)
                table_info.style = 'Table Grid'
                table_info.autofit = False

                # 添加表头
                header_cells = table_info.rows[0].cells
                header_cells[0].text = '字段名'
                header_cells[1].text = '数据类型'
                header_cells[2].text = '注释'

                # 添加字段信息
                for field in table_structure:
                    field_name = field[0]
                    data_type = field[1]
                    comment = field[8] if len(field) > 8 else ''
                    row_cells = table_info.add_row().cells
                    row_cells[0].text = field_name
                    row_cells[1].text = data_type
                    row_cells[2].text = comment
                print('', end='\r')
                print('转换进度：%.2f%%' % progress_rate, end='')
        save_filename = f'{connection_details.get("host").replace(".", "_")}_{connection_details.get("database")}'
        # document.save(f'{connection_details.get("host").replace(".", "_")}_{connection_details.get("database")}.docx')
    else:  # sqlite
        cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")  # 获取所有表名
        tables = cursor.fetchall()

        for i, table in enumerate(tables):
            progress_rate = round(((i + 1) / len(tables)) * progress_rate, 2)
            table_name = table[0]
            document.add_heading(table_name, level=2)

            cursor.execute(f"PRAGMA table_info({table_name})")  # 获取数据表结构
            table_structure = cursor.fetchall()

            table_info = document.add_table(rows=1, cols=3)
            table_info.style = 'Table Grid'
            table_info.autofit = False

            # 添加表头
            header_cells = table_info.rows[0].cells
            header_cells[0].text = '字段名'
            header_cells[1].text = '数据类型'
            header_cells[2].text = '注释'

            # 添加字段信息
            for field in table_structure:
                field_name = field[1]
                data_type = field[2]
                comment = field[8] if len(field) > 8 else ''
                row_cells = table_info.add_row().cells
                row_cells[0].text = field_name
                row_cells[1].text = data_type
                row_cells[2].text = comment
            print('', end='\r')
            print('转换进度：%.2f%%' % progress_rate, end='')
        # 保存Word文件
        if not save_filename:
            save_filename = os.path.split(connection_details)[-1].replace('.', '_')
    if save_filename.endswith('.docx'):
        document.save(save_filename)
    else:
        document.save(f'{save_filename}.docx')

# 调用函数
# if __name__ == '__main__':
#     # mysql
#     connection_details = {
#         'host': '192.168.0.240',
#         'port': 3306,
#         'user': 'chandao',
#         'password': '123456',
#         'database': 'chandao'
#     }
#     generate_data_dictionary('mysql', connection_details, 'xiaobai.docx')




#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/6/15 0:22
@File  : MonitorDriver2.py
'''
import os
import re
import subprocess
import copy
import time
import getpass
from lxml import etree
from adbutils import adb

USERHOMEDIR = f'C:\\Users\\{getpass.getuser()}'

UI_DUMP_FILE = f'{USERHOMEDIR}\\window_dump.xml'

def find_smallest_bounds_index(target, bounds_list):
    """
    查找包含目标点的最小边界的索引。

    参数：
        target (tuple): 目标点格式为 (x, y)。
        bounds_list (list): 边界列表，格式为 [[x0, y0, x1, y1], ...]。

    返回值：
        int: 包含目标点的最小边界的索引。
    """
    smallest_area = float('inf')
    smallest_index = None

    for i, bounds in enumerate(bounds_list):
        x0, y0, x1, y1 = bounds
        if x0 < target[0] < x1 and y0 < target[1] < y1:
            area = (x1 - x0) * (y1 - y0)
            if area < smallest_area:
                smallest_area = area
                smallest_index = i

    return smallest_index

def generate_xpath_with_attributes(element):
    if element is None:
        return ''
    if element.tag == 'hierarchy':
        class_value = element.get('class', '')
    else:
        class_value = element.get('class') if element.get('class') != '' else '*'
    resource_id = element.get('resource-id')
    text = element.get('text')

    if text:
        return f'//{class_value}[@text="{text}"]'
    elif resource_id:
        return f'//{class_value}[@resource_id="{resource_id}"]'
    else:
        parent = element.getparent()
        parent_xpath = generate_xpath_with_attributes(parent)
        return f'{parent_xpath}/{class_value}'


# 实时监控设备
def monitor_device(device, output_type: str = 'appium'):
    '''
    监控设备并输入指定类型的数据
    :param driverName: 设备名
    :param output_type: data or appium
    :return:
    '''
    device = adb.device(device.serial)
    app = device.app_current()
    package_name = getattr(app, 'package')
    activity_name = getattr(app, 'activity')
    print(f'''#! /usr/bin/env python
    
from appium import webdriver

caps = {{
    'automationName': 'UiAutomator2',
    'platformName': 'Android',
    'platformVersion': {device.shell(['getprop', 'ro.build.version.release']).strip()},
    'deviceName': {device.serial},
    'appPackage': {package_name},
    'appActivity': {activity_name},
    # 'noReset': True,
    # 'dontStopAppOnReset': True,
    'unicodeKeyboard': True,
    'resetKeyboard': True
}}

app = webdriver.Remote("http://127.0.0.1:4723/wd/hub", caps)

# 下面为定位表达式
''')
    deviceName = device.serial
    # 启动getevent命令
    event_cmd = f"adb -s {deviceName} shell getevent -lt /dev/input/event1"
    process = subprocess.Popen(event_cmd, stdout=subprocess.PIPE, shell=True)
    x = 0
    y = 0
    dx = copy.copy(x)
    dy = copy.copy(y)
    sj = ''
    status = [None, None]
    down_time = float(0)
    # 解析事件流
    while deviceName == device.serial:
        # 读取一行事件
        line = process.stdout.readline().decode().strip()
        # 获取x坐标
        if 'POSITION_X' in line:
            parts = line.split()
            x = int(parts[4], 16)

        if 'POSITION_Y' in line:
            parts = line.split()
            y = int(parts[4], 16)

        # 如果是点击事件
        if "BTN_TOUCH" in line and "DOWN" in line:
            down_time = time.time()
            status[0] = 'DOWN'
            # 记录点下时的坐标
            dx = copy.copy(x)
            dy = copy.copy(y)

            # 生成 XML 文件
            # os.popen('adb shell uiautomator dump')
            device.shell(['uiautomator', 'dump'])

        # 如果是点击事件
        if "BTN_TOUCH" in line and "UP" in line:
            hold_time = round(time.time() - down_time, 3)  # 秒
            status[1] = 'UP'
            # 记录点下时的坐标
            ux = copy.copy(x)
            uy = copy.copy(y)

            os.popen(f'adb pull /sdcard/window_dump.xml {UI_DUMP_FILE}')
            # device.shell(['pull', 'sdcard/window_dump.xml', UI_DUMP_FILE])
            while not os.path.exists(UI_DUMP_FILE) or os.path.getsize(UI_DUMP_FILE) == 0:
                try:
                    if os.path.exists(UI_DUMP_FILE):  os.remove(UI_DUMP_FILE)
                except PermissionError as e:
                    print('数据处理异常，正在重新尝试！')
                    # 将 XML 文件保存到本地
                    os.popen(f'adb pull /sdcard/window_dump.xml {UI_DUMP_FILE}')
                    continue
                finally:
                    # 将 XML 文件保存到本地
                    os.popen(f'adb pull /sdcard/window_dump.xml {UI_DUMP_FILE}')
            try:
                # 解析 XML 文档
                tree = etree.parse(UI_DUMP_FILE)
            except Exception as e:
                print('数据加载异常，正在重新尝试！')
                continue
            elements = tree.xpath('//node')
            bounds = [[int(num) for num in re.findall(r'\d+', s)] for s in tree.xpath('//node/@bounds')]
            # 防止点击页面以外的坐标，限制x与y的最大值与最小值
            x_page_max = bounds[0][2]
            y_page_max = bounds[0][3]
            ux = x_page_max-1 if ux > x_page_max else ux
            uy = y_page_max-1 if uy > y_page_max else uy
            dx = x_page_max-1 if dx > x_page_max else dx
            dy = y_page_max-1 if dy > y_page_max else dy
            ux = 1 if ux < 0 else ux
            uy = 1 if uy < 0 else uy
            dx = 1 if dx < 0 else dx
            dy = 1 if dy < 0 else dy
            up_index = find_smallest_bounds_index((ux, uy), bounds)# 查看是否再一个元素内，如果再就是点击
            down_index = find_smallest_bounds_index((dx, dy), bounds)

            if status == ['DOWN', 'UP']:
                if up_index == down_index:
                    if output_type == 'data':
                        sj = {
                            'platformName': 'Android',
                            'platformVersion': device.shell(['getprop', 'ro.build.version.release']).strip(),
                            'deviceName': device.serial,
                            'appPackage': getattr(device.app_current, 'package'),
                            'appActivity': getattr(device.app_current, 'activity'),
                            'action': 'click',
                            'xpath': generate_xpath_with_attributes(elements[up_index]),
                            'bounds': [dx, dy, ux, uy],
                            'hold_time': hold_time
                        }
                    elif output_type == 'appium':
                        # sj = f'app.find_element(By.XPATH, value="{elements[up_index].getroottree().getpath(elements[up_index])}").click()'      # /hierarchy/node/node[2]/node[2]/node/node/node/node/node/node[3]/node/node[1]/node[2]/node[1]
                        sj = f'app.find_element(By.XPATH, value="{generate_xpath_with_attributes(elements[up_index])}").click()'                                # /hierarchy/node/node[2]/node[2]/node/node/node/node/node/node[3]/node/node[1]/node[2]
                    else:
                        sj = '暂不支持此类型'
                else:
                    if output_type == 'data':
                        sj = {
                            'platformName': 'Android',
                            'platformVersion': device.shell(['getprop', 'ro.build.version.release']).strip(),
                            'deviceName': device.serial,
                            'appPackage': getattr(device.app_current, 'package'),
                            'appActivity': getattr(device.app_current, 'activity'),
                            'action': 'swipe',
                            'xpath': None,
                            'bounds': [dx, dy, ux, uy],
                            'hold_time': hold_time
                        }
                    elif output_type == 'appium':
                        sj = f'app.swipe({dx}, {dy}, {ux}, {uy}, {int(hold_time*1000)})'
                    else:
                        sj = '暂不支持此类型'
                status = [None, None]
                print(sj)

def cli():
    devices = []
    try:
        while not len(devices):
            print('等待设备连接中' + '.'*(int(time.time()) % 4), end='')
            devices = adb.device_list()
            time.sleep(1)
            print('', end='\r')
        monitor_device(devices[0], output_type='appium')
    except KeyboardInterrupt:
        pass



#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/21 0:25
@File  : test_xiaobai_shop_v2.py
'''
import pytest
from saf import full_load

''' 参数化 '''

data2 = {
    'test_login': {
        'keys': 'username, password, _assert',
        'values': [('xiaobai', '12345', 200), ('xiaohui', '1234567', 200)]
    }
}

data3 = full_load(open('..\\data\\testCase.yaml', 'r').read())

# 内部数据
@pytest.mark.parametrize('username, password, _assert', [('xiaobai', '12345', 200), ('xiaohui', '1234567', 200)])
def test_xiaobai_login1(username, password, _assert):
    # 业务实现代码
    assert _assert == 200

# 外部数据
@pytest.mark.parametrize(data2['test_login']['keys'], data2['test_login']['values'])
def test_xiaobai_login2(username, password, _assert):
    # 业务实现代码
    assert _assert == 200


# 外部文件数据
@pytest.mark.parametrize(data3['test_login']['keys'], [eval(v) for v in data3['test_login']['values']])
def test_xiaobai_login3(username, password, _assert):
    # 业务实现代码
    assert _assert == 200

if __name__ == '__main__':
    pytest.main(['-s', '-v', 'test_xiaobai_case_v2.py'])



#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : config.py
'''

# 执行环境、报告路径、日志路径、邮件信息
import sys
import os
from configparser import ConfigParser

CUR_ABS_PATH = os.path.dirname(os.path.realpath(__file__)) + '\\' if os.name == 'nt' else '/'

Config = ConfigParser()
Config.read(CUR_ABS_PATH + 'config.ini', encoding='UTF-8')

class host(object):
    @staticmethod
    def test():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def pro():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def current():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, Config.get(__class__.__name__, sys._getframe(0).f_code.co_name))

class report(object):
    @staticmethod
    def html_path():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return os.path.realpath(CUR_ABS_PATH + Config.get(__class__.__name__, sys._getframe(0).f_code.co_name))

    @staticmethod
    def allure_data_path():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return CUR_ABS_PATH + Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

class email(object):
    @staticmethod
    def smtp():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def port():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def to():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def username():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def password():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def subject():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def content():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)

    @staticmethod
    def files():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return os.path.realpath(CUR_ABS_PATH + Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)) + '\\' if os.name == 'nt' else '/'


class log(object):
    @staticmethod
    def path():
        '''
        __class__.__name__                  获取所在类的类名
        sys._getframe(0).f_code.co_name     获取所在函数的函数名
        :return:
        '''
        return os.path.realpath(CUR_ABS_PATH + Config.get(__class__.__name__, sys._getframe(0).f_code.co_name)) + '\\' if os.name == 'nt' else '/'



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/6/12 22:48
@File  : HTML2POM.py
'''
import re
from os import path, mkdir, remove
from bs4 import BeautifulSoup
from pypinyin import lazy_pinyin
from selenium import webdriver
from selenium.common.exceptions import NoSuchWindowException


class PageListener:
    def __init__(self, start_url: str = 'https://www.baidu.com', dirname: str = '.', rewrite: bool = True):
        '''
        基于Selenium基础操作过程中将每页内容转为POM
        :param to_str       : True将结果转为字符串返回 ,False将结果写入到脚本，自动输出到Pages包中
        '''
        if rewrite and path.exists(f'{dirname}/Pages'):
            try:
                remove(f'{dirname}/Pages')
            except PermissionError as e:
                pass
        if not path.exists(f'{dirname}/Pages'):
            mkdir('Pages')
        if not path.exists(f'{dirname}/Pages/__init__.py'):
            with open(f'{dirname}/Pages/__init__.py', 'w', encoding='UTF-8') as f:
                f.write('''#! /usr/bin/env python
                
#********************************#
#    欢迎使用自动生成POM代码工具      #
#      Auther:xiaobaiTser        #
#********************************#

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
                ''')
                f.close()

        driver = webdriver.Chrome()
        driver.get(start_url)
        driver.implicitly_wait(30)
        new_title = ''.join(re.findall('\w+', driver.title))
        self.code2file(
            code=self.identify_inputs_and_buttons(driver.current_url, driver.page_source),
            filename=f'{dirname}/Pages/{"_".join(lazy_pinyin(new_title)).title()}.py')
        # 监视浏览器URL变化、标签页变化
        old_url = driver.current_url
        old_handles = driver.window_handles
        while True:
            try:
                new_url = driver.current_url
                new_handles = driver.window_handles
                if len(new_handles) > len(old_handles):
                    driver.switch_to.window(list(set(new_handles)-set(old_handles))[0])
                    new_title = ''.join(re.findall('\w+', driver.title))
                    self.code2file(
                        code=self.identify_inputs_and_buttons(driver.current_url, driver.page_source),
                        filename=f'{dirname}/Pages/{"_".join(lazy_pinyin(new_title)).title()}.py')
                    old_handles = new_handles
                elif new_url != old_url:
                    new_title = ''.join(re.findall('\w+', driver.title))
                    self.code2file(
                        code=self.identify_inputs_and_buttons(driver.current_url, driver.page_source),
                        filename=f'{dirname}/Pages/{"_".join(lazy_pinyin(new_title)).title()}.py')
            except KeyboardInterrupt as e:
                exit(-1)
            except NoSuchWindowException as e:
                exit(-2)

    @classmethod
    def code2file(cls, code: str, filename: str = None):
        with open(filename, 'w', encoding='UTF-8') as f:
            f.write(code)
            f.close()
            del f

    @classmethod
    def identify_inputs_and_buttons(cls, url, html):
        '''
        1、解析HTML获取输入框与按钮并获取xpath表达式
        2、将输入框与按钮转为POM代码，一个页面单独一个脚本，一个脚本单独一个类
        :param html:
        :return:
        '''
        soup = BeautifulSoup(html, 'html.parser')
        find_all_input = soup.find_all(['input', 'textarea'])
        find_all_button = soup.find_all('button')
        find_all_button.extend(soup.find_all('a'))
        find_all_button.extend(soup.find_all('input', attrs={'type': ['button', 'submit']}))
        input_list = []
        button_list = []
        for input_tag in find_all_input:
            if input_tag not in soup.find_all('input', attrs={'type': ['button', 'submit', 'hidden']}):
                input_name = input_tag.get('name') or input_tag.name
                input_xpath = cls.get_xpath(input_tag)
                if input_name:
                    input_list.append({'tag': input_tag, 'name': input_name, 'xpath': input_xpath})
        for button_tag in find_all_button:
            button_name = button_tag.get('name') or button_tag.text.strip() or button_tag.name
            button_xpath = cls.get_xpath(button_tag)
            button_list.append({'tag': button_tag, 'name': button_name, 'xpath': button_xpath})
        title = '_'.join(lazy_pinyin(soup.select("title")[0].text)).upper()
        title = ''.join(re.findall('[0-9a-zA-Z_]+', title))
        return cls.converter(page_name=title, url=url,
                              input_list=input_list, button_list=button_list)

    @classmethod
    def get_xpath(cls, element):
        components = []
        child = element
        while child is not None:
            siblings = child.find_previous_siblings()
            index = len(siblings) + 1
            if child.name == 'html':
                components.insert(0, '/html')
                break
            if child.name == 'body':
                components.insert(0, '/body')
                break
            components.insert(0, f'/{child.name}[{index}]')
            child = child.parent
        xpath = ''.join(components)
        xpath = xpath if xpath.startswith('/html') else '/' + xpath
        return xpath

    @classmethod
    def converter(cls, page_name: str, url: str, input_list: list, button_list: list):
        function_strings = []
        function_names = []
        function_strings.append('#! /usr/bin/env python')
        function_strings.append(f'')
        function_strings.append('#********************************#')
        function_strings.append('#    欢迎使用自动生成POM代码工具      #')
        function_strings.append('#      Auther:xiaobaiTser        #')
        function_strings.append('#********************************#')
        function_strings.append(f'')
        function_strings.append('from selenium.webdriver.common.by import By')
        function_strings.append(f'')
        function_strings.append(f'class {page_name}:')
        function_strings.append('\tdef __init__(self, driver):')
        function_strings.append(f'\t\t# 当前URL: {url}')
        function_strings.append('\t\tself.driver = driver')
        function_strings.append(f'')
        for input_item in input_list:
            function_name = "_".join(lazy_pinyin(input_item['name']))
            function_name = ''.join(re.findall('[0-9a-zA-Z_]+', function_name))
            function_names.append(function_name)
            if function_names.count(function_name) > 1:
                function_name = f'{function_name}_{function_names.count(function_name)-1}'
            xpath = input_item['xpath']
            function_strings.append(f'\tdef send_{function_name}(self, data):')
            function_strings.append("\t\t'''")
            function_strings.append('\t\t当前元素：')
            input_item['tag'] = str(input_item['tag']).replace('\n', '\n\t\t')
            function_strings.append(f"\t\t{input_item['tag']}")
            function_strings.append("\t\t'''")
            function_strings.append(f'\t\tself.driver.find_element(By.XPATH, "{xpath}").send_keys(data)')
            function_strings.append(f'')
        for button_item in button_list:
            function_name = "_".join(lazy_pinyin(button_item['name']))
            function_name = ''.join(re.findall('[0-9a-zA-Z_]+', function_name))
            function_names.append(function_name)
            if function_names.count(function_name) > 1:
                function_name = f'{function_name}_{function_names.count(function_name)-1}'
            xpath = button_item['xpath']
            function_strings.append(f'\tdef click_{function_name}(self):')
            function_strings.append("\t\t'''")
            function_strings.append('\t\t当前元素：')
            button_item['tag'] = str(button_item['tag']).replace('\n', '\n\t\t')
            function_strings.append(f"\t\t{button_item['tag']}")
            function_strings.append("\t\t'''")
            function_strings.append(f'\t\tself.driver.find_element(By.XPATH, "{xpath}").click()')
            function_strings.append(f'')
        return '\n'.join(function_strings)

# if __name__ == '__main__':
#     PageListener()


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : ExcelUtils.py
'''

import xlrd

def read_excel(file_path):
    workbook = xlrd.open_workbook(file_path)
    sheet = workbook.sheet_by_index(0)  # 假设用例在第一个sheet中

    # 获取标题列、操作数据列和预期结果列的索引
    title_col_index = None
    data_col_index = None
    expected_result_col_index = None

    for col_index in range(sheet.ncols):
        cell_value = sheet.cell_value(0, col_index)
        if cell_value == "标题列":
            title_col_index = col_index
        elif cell_value == "操作数据列":
            data_col_index = col_index
        elif cell_value == "预期结果列":
            expected_result_col_index = col_index

    if title_col_index is None or data_col_index is None or expected_result_col_index is None:
        raise ValueError("未找到标题列、操作数据列或预期结果列")

    # 读取用例数据
    test_cases = []
    for row_index in range(1, sheet.nrows):
        title = sheet.cell_value(row_index, title_col_index)
        data = sheet.cell_value(row_index, data_col_index)
        expected_result = sheet.cell_value(row_index, expected_result_col_index)
        test_cases.append((title, data, expected_result))

    return test_cases



#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/9/2 1:12
@File  : test_case.py
'''

import pytest

@pytest.mark.parametrize('a', [1])
@pytest.mark.parametrize('b', [1])
def test_a(a, b):
    print(a, b)


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : test_one_html_case_template.py
'''

import os
import pytest
from loguru import logger
from saf.utils.YamlUtils import yamlUtils
from saf.example.api.Config.config import log

''' 初始化日志 '''
logger.add(log.path() + 'info.log', level='INFO', rotation='1 days', retention='7 days', enqueue=True, encoding='UTF-8')
logger.add(log.path() + 'error.log', level='ERROR', rotation='1 days', retention='7 days', enqueue=True, encoding='UTF-8')

''' 获取当前脚本运行脚本的绝对路径，避免环境不同导致的脚本运行错误 '''
CUR_ABS_PATH = os.path.dirname(os.path.realpath(__file__)) + '\\' if os.name == 'nt' else '/'
YAML_FILE_PATH = os.path.realpath(CUR_ABS_PATH + '../data/') + '\\' if os.name == 'nt' else '/'

def setup_module(): pass

def teardown_module(): pass

class TestClass(object):
    ''' 解析测试数据文件*.yml|*.yaml '''
    case_list = [YAML_FILE_PATH + f for f in os.listdir(YAML_FILE_PATH) if os.path.splitext(f)[1] in ['.yml', '.yaml']]

    @pytest.mark.parametrize('filename', case_list)
    def test_case(self, filename):
        ''' 解析YAML文件 '''
        result = yamlUtils.read(files=filename)
        logger.info(f'{filename}解析结果：{result}')

        ''' 执行测试 '''
        logger.info('开始执行用例数据...')


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : send_email_script.py
'''

from xiaobaiauto2.utils.xiaobaiauto2Email import xemail
from saf.example.api.Config.config import email


user = xemail(smtp_server=email.smtp(),
               smtp_port=email.port(),
               username=email.username(),
               password=email.password())

user.send(to=email.to(),
          subject=email.subject(),
          content=email.content(),
          files=email.files())


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : YamlUtils.py
'''

''' 解析 *.YAML / *.YML文件 '''
from yaml import full_load
from typing import Union
from os import path as os_path, listdir, name

class yamlUtils(object):
    @staticmethod
    def read(files: Union[str, list] = None, dir_path: Union[str] = None):
        '''解析指定yaml文件(或者指定文件夹内的所有yaml文件)
        :param files: 指定单个（*.yml/*.yaml）文件或者存储在列表中的多个（*.yml/*.yaml）文件
        :param dir_path:  指定文件夹路径，解析其内的所有（*.yml/*.yaml）文件
        实例：
            yamlUtils.read(files='a.yml')
            yamlUtils.read(files='a.yaml')
            yamlUtils.read(files=['a.yml','b.yml'])
            yamlUtils.read(files=['a.yml','b.yaml'])
            yamlUtils.read(dir_path='../data')
            yamlUtils.read(dir_path='D:\\data')
        '''
        if files:
            if type(files) == str and os_path.isfile(files) and os_path.splitext(files)[1] in ['.yml', '.yaml']:
                ''' 单文件场景 '''
                files = [files]
            elif type(files) == list:
                ''' 多文件场景 '''
                ''' 过滤非法文件 '''
                files = [i for i in files if os_path.splitext(i)[1] in ['.yml', '.yaml'] and os_path.isfile(i)]
            else:
                raise TypeError('您输入的内容不支持！')
        elif dir_path:
            ''' 提供文件夹路径场景 '''
            step = '\\' if name == 'nt' else '/'
            files = [dir_path + step + i for i in listdir(dir_path) if os_path.splitext(i)[1] in ['.yml', '.yaml']]
        else:
            ''' 参数异常场景 '''
            raise ValueError('参数值有误！')
        if len(files) > 0:
            result = list()
            for file in files:
                result.append(full_load(open(file, 'r', encoding='UTF-8')))
            return result
        return []


from loguru import logger
from datetime import datetime

logger.add(f"logs/web_{datetime.now().strftime('%Y_%m_%d')}.log", rotation="1 day", enqueue=True, encoding='UTF-8')


def log_info(message):
    logger.info(message)

def log_error(message):
    logger.error(message)



#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/24 2:26
@File  : xiaobaicmd.py
'''
import os.path
from shutil import copytree
import click
from saf.utils.selenium2POM import PageListener
from saf.utils.MonitorAndroidPackageGUI import gui
from saf.utils.MonitorAndroidPackageCLI import cli
from saf.utils.MonitorAndroidDeviceGUI import MonitorDevice
from saf.utils.MonitorAndroidPackagePower import power

@click.command()
@click.option('--template', '-t', type=click.Choice(['web', 'api', 'app']), nargs=1, help='创建自动化项目模板')
@click.option('--dirname', '-d', default='.', type=str, nargs=1, help='创建自动化项目模板存放的目录')
@click.option('--url', '-u', type=str, nargs=1, help='自动生成PageObject代码的首个URL')
@click.option('--rewrite', '-r', default=1, type=int, nargs=1, help='自动生成PageObject是否重写')
@click.option('--monitor', '-m', type=click.Choice(['gui', 'cli', 'power']), nargs=1, help='自动识别APP页面点击/滑动的Xpath表达式与坐标信息')
@click.option('--device', '-e', default=1, type=int, nargs=1, help='监控设备的界面图像，从1开始计算设备序号，暂无其他功能')
def main(template, dirname, url, rewrite, monitor, device):
    if template:
        if "web" == template.lower():
            copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), r'..\web'),
                     os.path.join(os.path.abspath(dirname), 'web')
                     )
        elif "api" == template.lower():
            copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), r'..\api'),
                     os.path.join(os.path.abspath(dirname), 'api')
                     )
        elif "app" == template.lower():
            copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), r'..\app'),
                     os.path.join(os.path.abspath(dirname), 'app')
                     )
        else:
            raise ("您输入的数据有误，有效范围：web 或 api 或 app")
    elif url:
        if rewrite:
            PageListener(start_url=url, dirname=dirname, rewrite=True)
        else:
            PageListener(start_url=url, dirname=dirname, rewrite=False)
    elif monitor == "gui":
        gui()
    elif monitor == "cli":
        cli()
    elif monitor == "power":
        power()
    elif device:
        MonitorDevice(device - 1)


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@email : 807447312@qq.com
@Time  : 2023/6/28 20:41
@File  : MonitorAndroidPackagePowser.py
'''
import os
import tkinter as tk
import subprocess
import threading
import time
from tkinter import filedialog, messagebox

import matplotlib.pyplot as plt
import csv

from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

class App:
    def __init__(self, master):
        CUR_PATH = os.path.dirname(os.path.abspath(__file__))
        self.master = master
        master.title('Android电量监控')
        master.geometry('700x500')
        master.iconbitmap(CUR_PATH + '\\..\\data\\favicon.ico')

        self.device_id = None
        # self.package_name = None
        self.power_data = []
        self.is_running = False

        # 创建开始、停止、导出按钮
        self.start_button = tk.Button(master, text='开始', command=self.start)
        self.start_button.pack(side='left', padx=10, pady=10)

        self.stop_button = tk.Button(master, text='停止', command=self.stop)
        self.stop_button.pack(side='left', padx=10, pady=10)

        self.export_button = tk.Button(master, text='导出', command=self.export)
        self.export_button.pack(side='left', padx=10, pady=10)

        # 创建tkinter标签，用于显示最新的电量数据
        self.label = tk.Label(master, text='当前电量：')
        self.label.pack(side='bottom', padx=10, pady=10)

        # 创建matplotlib图形区域
        self.fig = plt.figure(figsize=(6, 4))
        self.ax = self.fig.add_subplot(111)
        self.ax.set_xlabel('Time(s)')
        self.ax.set_ylabel('Power(%)')
        self.ax.set_xlim(0, 30)
        self.ax.set_ylim(0, 100)

        # 创建tkinter绘图区域
        self.canvas = FigureCanvasTkAgg(self.fig, master=master)
        self.canvas.get_tk_widget().pack(side='top', fill='both', expand=1)

    def start(self):
        self.start_time = time.time()
        if not self.is_running:
            self.is_running = True
            self.device_id = subprocess.check_output(['adb', 'devices']).decode('utf-8').split('\n')[1].split('\t')[0]
            # self.package_name = subprocess.check_output(['adb', 'shell', 'dumpsys', 'activity', 'activities']).decode('utf-8').split('\n')[0].split()[1]
            self.power_data = []

            # 在后台启动一个线程来获取电量数据
            self.power_thread = threading.Thread(target=self.get_power_data)
            self.power_thread.start()

    def stop(self):
        self.is_running = False

    def export(self):
        # 弹出文件对话框，获取用户选择的文件名和存储位置
        file_path = filedialog.asksaveasfilename(defaultextension='.csv', filetypes=(("CSV文件", "*.csv"),))
        if file_path:
            try:
                # 导出电量数据到CSV文件
                with open(file_path, 'w', newline='') as f:
                    writer = csv.writer(f)
                    writer.writerows([('时间（秒）', '电量（%）')])
                    writer.writerows(self.power_data)
                messagebox.showinfo("小白提醒：", "数据已经保存完成！")
            except PermissionError:
                messagebox.showerror("小白提示：", "文件无法访问！文件是否打开？请先关闭后重新尝试！")

    def get_power_data(self):
        while self.is_running:
            output = subprocess.check_output(['adb', '-s', self.device_id, 'shell', 'dumpsys', 'battery', '|', 'grep', 'level']).decode('utf-8').strip()
            power_level = int(output.split(':')[1])
            self.power_data.append((int(time.time()-self.start_time), power_level))
            self.update_power_data()
            time.sleep(1)

    def update_power_data(self):
        # 更新图表数据和电量标签
        if self.power_data:
            self.ax.clear()
            self.ax.set_xlabel('Time(s)')
            self.ax.set_ylabel('Power(%)')
            if self.power_data[-1][0] > 30:
                self.ax.set_xlim(max(0, self.power_data[-1][0] - 30), self.power_data[-1][0] + 1)
            else:
                self.ax.set_xlim(0, 30)
            self.ax.set_ylim(0, 100)

            if len(self.ax.lines) > 0:
                self.ax.lines.pop(0)
            if self.power_data[-1][0] > 30:
                self.ax.plot([t for t, _ in self.power_data[-30:]], [p for _, p in self.power_data[-30:]])
            else:
                self.ax.plot([t for t, _ in self.power_data], [p for _, p in self.power_data])
            self.canvas.draw()
            # 更新电量标签
            current_power = self.power_data[-1][1]
            self.label.config(text=f'当前电量：{current_power}%')

def power():
    root = tk.Tk()
    app = App(root)
    root.mainloop()

# if __name__ == '__main__':
#     power()


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/6/19 22:01
@File  : MonitorAndroidDeviceGUI.py
'''

import tkinter as tk
from PIL import Image, ImageTk
from io import BytesIO
from adbutils import adb
import os

class MonitorAndroid:
    def __init__(self, device):
        CUR_PATH = os.path.dirname(os.path.abspath(__file__))
        # 创建Tkinter界面
        self.root = tk.Tk()
        self.root.iconbitmap(CUR_PATH + '\\..\\data\\favicon.ico')

        # 获取设备的分辨率并设置Tkinter界面的大小
        self.device = device
        width, height = device.window_size()
        self.root.geometry(f'{width // 5}x{height // 5}')
        self.root.resizable(False, False)
        # 标题
        self.root.title(device.serial)

        self.label = tk.Label(self.root)
        self.label.pack()
        # 开始更新图片
        self.device_screenshot()
        self.root.mainloop()

    def device_screenshot(self):
        # 获取Android设备的屏幕截图
        screenshot = self.device.screenshot()

        # 将截图转换为字节流
        byte_stream = BytesIO()
        screenshot.save(byte_stream, format='PNG')
        screenshot_byte = byte_stream.getvalue()

        # 将字节流转换为Tkinter可以使用的格式
        image = Image.open(BytesIO(screenshot_byte))
        new_size = (image.width // 5, image.height // 5)
        resized_image = image.resize(new_size)
        photo = ImageTk.PhotoImage(resized_image)

        # 更新Tkinter界面的图片
        self.label.config(image=photo)
        self.label.image = photo

        # 每100毫秒更新一次图片
        self.label.after(100, self.device_screenshot)

def MonitorDevice(index: int = 0):
    devices = adb.device_list()
    print(f'设备列表：{devices}，您选择的是第{index+1}个')
    if len(devices) > 0 and index < len(devices):
        MonitorAndroid(device=devices[index])
    else:
        print('您选择了无效的序号！')


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/9/4 0:50
@File  : downloadUtils.py
'''
''' 下载资源 '''

import os
import requests
import threading
from tkinter import *
from tkinter import filedialog

class Downloader:
    def __init__(self, url, num_threads, save_path):
        self.url = url
        self.num_threads = num_threads
        self.save_path = save_path
        self.filename = url.split('/')[-1]
        self.file_size = int(requests.head(self.url).headers['Content-Length'])

    def download(self, start, end, filename):
        if os.path.exists(filename):
            first_byte = os.path.getsize(filename)
        else:
            first_byte = start
        if first_byte >= end:
            return

        headers = {'Range': 'bytes=%d-%d' % (first_byte, end)}
        r = requests.get(self.url, headers=headers, stream=True)

        with open(filename, 'ab') as fp:
            fp.write(r.content)

    def multi_thread_download(self):
        part = self.file_size // self.num_threads
        for i in range(self.num_threads):
            start = part * i
            if i == self.num_threads - 1:
                end = self.file_size
            else:
                end = start + part
            t = threading.Thread(target=self.download, args=(start, end, self.save_path + '/part%d' % i))
            t.setDaemon(True)
            t.start()

        main_thread = threading.current_thread()
        for t in threading.enumerate():
            if t is main_thread:
                continue
            t.join()
        self.merge_files()

    def merge_files(self):
        with open(self.save_path + '/' + self.filename, 'wb') as fp:
            for i in range(self.num_threads):
                with open(self.save_path + '/part%d' % i, 'rb') as f:
                    fp.write(f.read())
                os.remove(self.save_path + '/part%d' % i)

def download_file():
    url = url_entry.get()
    save_path = path_entry.get()
    num_threads = int(thread_entry.get())
    downloader = Downloader(url, num_threads, save_path)
    downloader.multi_thread_download()

def browse_directory():
    directory = filedialog.askdirectory()
    path_entry.delete(0, END)
    path_entry.insert(0, directory)

root = Tk()
root.title('Downloader')

url_label = Label(root, text='URL:')
url_label.grid(row=0, column=0)
url_entry = Entry(root, width=50)
url_entry.grid(row=0, column=1)

path_label = Label(root, text='Save Path:')
path_label.grid(row=1, column=0)
path_entry = Entry(root, width=50)
path_entry.grid(row=1, column=1)
path_button = Button(root, text='Browse', command=browse_directory)
path_button.grid(row=1, column=2)

thread_label = Label(root, text='Number of Threads:')
thread_label.grid(row=2, column=0)
thread_entry = Entry(root, width=50)
thread_entry.grid(row=2, column=1)

download_button = Button(root, text='Download', command=download_file)
download_button.grid(row=3, column=1)

root.mainloop()




#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : __init__.py
'''

import os
def CUR_ABS_PATH():
    ''' 返回 当前运行脚本的绝对路径 '''
    return os.path.dirname(os.path.realpath(__file__)) + '\\' if os.name == 'nt' else '/'


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : api_client.py
'''

import requests

class APIClient:
    def __init__(self, base_url):
        self.base_url = base_url

    def get(self, endpoint, headers=None, params=None, **kwargs):
        url = self.base_url + endpoint
        response = requests.get(url, headers=headers, params=params, **kwargs)
        return response

    def post(self, endpoint, headers=None, data=None, json=None, **kwargs):
        url = self.base_url + endpoint
        response = requests.post(url, headers=headers, data=data, json=json, **kwargs)
        return response

    def put(self, endpoint, headers=None, data=None, json=None, **kwargs):
        url = self.base_url + endpoint
        response = requests.put(url, headers=headers, data=data, json=json, **kwargs)
        return response

    def delete(self, endpoint, headers=None, **kwargs):
        url = self.base_url + endpoint
        response = requests.delete(url, headers=headers, **kwargs)
        return response




#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/24 22:05
@File  : test_xiaobai_allure_shop.py
'''
import pytest
import allure

@allure.feature('下单')
class Test_order():
    @allure.story('登录')
    def test_login(self):
        ''' 登录 '''
        with allure.step('输入账户'):
            assert True
        with allure.step('输入密码'):
            assert True
        with allure.step('点击登录'):
            assert True

    @allure.story('搜索商品')
    def test_search(self):
        ''' 搜索商品 '''
        with allure.step('搜索框输入：苹果'):
            assert True
        with allure.step('点击搜索按钮'):
            assert False


'''
# 执行脚本
pytest test_xiaobai_case_allure.py --alluredir=../data

# 打开报告
allure serve ../data
或者
allure generate -c -o ../report ../data
allure open ../report
'''


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:47
@File  : __init__.py
'''
'''
PO模式：
    1、基础元素定位函数的封装（基础定位、键盘、鼠标事件，解决：定位不稳定问题）
    2、页面元素的定位（定位层）
    3、页面元素的操作（操作层）
    4、界面业务的操作（业务层）
    5、基于单元模块的用例（用例层）
'''



#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Email : 807447312@qq.com
@Time  : 2023/1/13 23:47
@File  : run.py
'''

import os
import pytest
from loguru import logger
from saf.example.api.Config.config import report, log

logger.add(log.path() + 'info.log', level='INFO', rotation='1 days', retention='7 days', enqueue=True, encoding='UTF-8')

if __name__ == '__main__':
    ''' 执行用例之前清理上次输出的测试报告文件 '''
    if os.path.isfile(report.html_path()):
        os.remove(report.html_path())
        logger.info(f'已经移除{report.html_path()}')

    ''' 批量执行TestCases目录下的所有测试用例 '''
    pytest.main(['-s', f'--html={report.html_path()}', '--self-contained-html', 'TestCases'])

    ''' 如果测试脚本之间不存在数据关联，可以使用多进程并发运行:-n=4  4表示四个进程 '''
    # pytest.main(['-s', '-n=4', f'--html={report.html_path()}', '--self-contained-html', 'TestCases'])

    ''' 测试用例执行结束之后，发送邮件 '''
    if os.path.isfile(report.html_path()):
        # os.system('python send_email_script.py')
        logger.info('含测试报告文件的邮件已发送')